-- lua/gemini/api.lua

local M = {}

-- Store conversation history
local conversation_history = {}

-- Helper function to get API key
local function get_api_key()
	return vim.g.gemini_api_key or os.getenv("GEMINI_API_KEY")
end

-- Create a Request class to handle HTTP requests
local Request = {}
Request.__index = Request

function Request.new(url, payload)
	local self = setmetatable({}, Request)
	self.url = url
	self.payload = payload
	self.response = ""
	self.error_msg = ""
	self.stdout = vim.loop.new_pipe()
	self.stderr = vim.loop.new_pipe()
	self.handle = nil
	return self
end

function Request:cleanup()
	if self.stdout then self.stdout:close() end
	if self.stderr then self.stderr:close() end
	if self.handle then self.handle:close() end
end

function Request:handle_error(callback, msg)
	self:cleanup()
	vim.schedule(function()
		callback(nil, msg)
	end)
end

function Request:setup_pipes(callback)
	-- Handle stdout data
	self.stdout:read_start(function(err, chunk)
		if err then
			self:handle_error(callback, "Read error: " .. err)
			return
		end
		if chunk then
			self.response = self.response .. chunk
		end
	end)

	-- Handle stderr data
	self.stderr:read_start(function(err, chunk)
		if err then
			self:handle_error(callback, "Error reading stderr: " .. err)
			return
		end
		if chunk then
			self.error_msg = self.error_msg .. chunk
		end
	end)
end

function Request:execute(callback)
	self.handle = vim.loop.spawn('curl', {
		args = {
			'-s',
			'-X', 'POST',
			'-H', 'Content-Type: application/json',
			'-d', self.payload,
			self.url
		},
		stdio = {nil, self.stdout, self.stderr}
	}, function(exit_code)
		self:cleanup()

		if exit_code ~= 0 then
			vim.schedule(function()
				callback(nil, "Curl failed with code " .. exit_code .. ": " .. self.error_msg)
			end)
			return
		end

		-- Try to parse the response
		local success, decoded = pcall(vim.json.decode, self.response)
		vim.schedule(function()
			if success then
				callback(decoded)
			else
				callback(nil, "JSON decode error: " .. self.response)
			end
		end)
	end)

	if not self.handle then
		self:handle_error(callback, "Failed to start curl")
		return
	end

	self:setup_pipes(callback)
end

-- Message handling functions
local Message = {}

function Message.create_contents(prompt, context)
	local contents = {}
	
	-- Add conversation history
	for _, message in ipairs(conversation_history) do
		table.insert(contents, {
			role = message.role,
			parts = {{text = message.content}}
		})
	end

	-- Add current prompt
	if context then
		table.insert(contents, {
			role = "user",
			parts = {{text = "Context:\n" .. context .. "\n\nQuery: " .. prompt}}
		})
	else
		table.insert(contents, {
			role = "user",
			parts = {{text = prompt}}
		})
	end

	return contents
end

function Message.store_message(role, content)
	table.insert(conversation_history, {role = role, content = content})
end

function Message.handle_response(result, callback)
	if result.error then
		callback(nil, "API Error: " .. vim.inspect(result.error))
		return
	end

	if result.candidates and
	   result.candidates[1] and
	   result.candidates[1].content and
	   result.candidates[1].content.parts and
	   result.candidates[1].content.parts[1] and
	   result.candidates[1].content.parts[1].text then
		
		local response_text = result.candidates[1].content.parts[1].text
		Message.store_message("model", response_text)
		callback(response_text)
	else
		callback(nil, "Unexpected response structure")
	end
end

-- Main API functions
function M.get_response(prompt, context, callback)
	local api_key = get_api_key()
	
	if not api_key then
		vim.schedule(function()
			callback(nil, "API key not set")
		end)
		return
	end

	-- Store prompt in history
	Message.store_message("user", prompt)

	local contents = Message.create_contents(prompt, context)
	local payload = vim.json.encode({contents = contents})
	local url = string.format(
		"https://generativelanguage.googleapis.com/v1/models/%s:generateContent?key=%s",
		"gemini-2.0-flash",
		api_key
	)

	local request = Request.new(url, payload)
	request:execute(function(result, error)
		if error then
			callback(nil, error)
			return
		end
		Message.handle_response(result, callback)
	end)
end

function M.clear_conversation()
	conversation_history = {}
end

return M
